﻿; *****************************************************************************
; MemDLL.pbi
; by Luis
; http://luis.no-ip.net
;
; Load a DLL from memory.
; OS: Windows
; Build: x86, x64
; Requires: nothing
;
; v1.0  Initial release                                April 1, 2016    PB 5.42
; *****************************************************************************

; With this module is possible to load a DLL from memory instead of from file.
; Tested on Windows 7 x64 with just *ONE* DLL both in 32 and 64 bit flavours, so probably more testing is required.
; Worked apparently well for the limited use I made of it.

; Thanks to:

; Joachim Bauch (http://www.joachim-bauch.de) for the original code I studied to learn something about what a loader should do.
; The user "shebaw" (http://www.rohitab.com forum) for some nice C snippets I converted to PB.
; Many Microsoft articles and text files describing the PE format.
; The x64dbg debugger (http://x64dbg.com).

; What this loader does:

; 1) Check if the image in memory looks like a valid DLL.
; 2) Reserve enough virtual address space to map the DLL to it, at the preferred address specified in the DLL if possible, else at a different one.
; 3) Map the DLL to virtual space, copies the headers and the sections in memory to the appropriate aligned starting offsets.
; 4) Process the import table, load the required DLLs (dependencies) and patch the table with the loaded function addresses.
; 5) If any of functions is forwarded to another DLL, the target DLL module is retrieved if the DLL is already in memory, else the required DLL is loaded.
; 6) If the DLL has not been mapped to the preferred location, the relocation table is processed and relocations are applied.
; 7) The protection flags specified in the section headers are applied to the sections in memory.
; 8) The #DLL_PROCESS_ATTACH event is signaled to the DLL and its handle is returned.

; Limitations:

; It's not possible to load from memory a DLL "A" depending on a DLL "B" if "B" is also embedded in memory and not available in the filesystem.
; It's possible to enhance the code to support even that but it's a little extreme and I don't need it.
; In other words you can:
; - Load all the DLLs you want from memory as long they are not dependent on each other.
; - Load all the DLLs you want from memory as long their dependencies are available in the file system (tipically OS DLLs : GDI32, KERNEL32, MSVCRT, ...)

; The idea is to embed just 1 or 2 DLLs in your program since sometimes may be annoying to have to distribute them separately (for example for a small techdemo, intro, utility etc.)
; If your program requires MANY DLLs probably is better to just distribute them as external files along with your program.
; In any case if they are not dependent on each other you can embed them all if you really want.

; Usage example:

; IncludeFile "MemDLL.pbi"
;
; DataSection
;  glfw3:
;  IncludeBinary "glfw3.x86.dll"
; EndDataSection
;
; hglfw = MemDLL_LoadLibrary(?glfw3)
; 
; If hglfw
;     PrototypeC.i glfwInit()     
;     Define glfwInit.glfwInit = MemDLL_GetProcAddress(hglfw, "glfwInit")
;       ...
;     MemDLL_FreeLibrary(hglfw)
; EndIf


;{ Some annotations about the PE structure 

; +- [IMAGE_DOS_HEADER] --------------------------------------------------------+
; +     e_magic.w ($5A4D) 'MZ'
; +     e_cblp.w
; +     e_cp.w
; +     e_crlc.w
; +     e_cparhdr.w
; +     e_minalloc.w
; +     e_maxalloc.w
; +     e_ss.w
; +     e_sp.w
; +     e_csum.w
; +     e_ip.w
; +     e_cs.w
; +     e_lfarlc.w
; +     e_ovno.w
; +     e_res.w[4]
; +     e_oemid.w
; +     e_oeminfo.w
; +     e_res2.w[10]
; +     e_lfanew.l -> points to IMAGE_NT_HEADERS signature
; +-----------------------------------------------------------------------------+

; +                                DOS STUB
; +-----------------------------------------------------------------------------+
; + DOS executable printing "This program cannot be run in DOS mode"             
; +-----------------------------------------------------------------------------+

;                       PE HEADER = IMAGE_NT_HEADERS =
;           Signature + IMAGE_FILE_HEADER + IMAGE_OPTIONAL_HEADER

; +- [IMAGE_NT_HEADERS] --------------------------------------------------------+
; +     Signature.l ; $00004550 'PE00'
; +- [IMAGE_FILE_HEADER] -------------------------------------------------------+
; +     Machine.w
; +     NumberOfSections.w ; This indicates the size of the section table, which immediately follows the headers.
; +     TimeDateStamp.l
; +     PointerToSymbolTable.l
; +     NumberOfSymbols.l
; +     SizeOfOptionalHeader.w
; +     Characteristics.w ; Flags that indicate attributes of the object or image file.
; +- [IMAGE_OPTIONAL_HEADER] ---------------------------------------------------+
; + (COFF FIELDS) 
; +     Magic.w
; +     MajorLinkerVersion.b
; +     MinorLinkerVersion.b
; +     SizeOfCode.l
; +     SizeOfInitializedData.l ; Size of the initialized data section, or the sum of all such sections if there are multiple data sections.
; +     SizeOfUninitializedData.l ; Size of the uninitialized data section, or the sum of all such sections if there are multiple sections.
; +     AddressOfEntryPoint.l ; VRA of the entry point (optional for DLLs)
; +     BaseOfCode.l
; +     BaseOfData.l
; + (WINDOWS FIELDS)
; +     ImageBase.i ; The preferred address of the first byte of image when loaded into memory (in other words, of a module)
; +     SectionAlignment.l ; The alignment of sections when they are loaded in memory. It must be greater than or equal to FileAlignment. The default is the page size.
; +     FileAlignment.l ; The alignment used for sections in the image file. If the SectionAlignment is less than page size, then FileAlignment must match SectionAlignment.
; +     MajorOperatingSystemVersion.w
; +     MinorOperatingSystemVersion.w
; +     MajorImageVersion.w
; +     MinorImageVersion.w
; +     MajorSubsystemVersion.w
; +     MinorSubsystemVersion.w
; +     Win32VersionValue.l
; +     SizeOfImage.l  ; The amount of space to reserve in the virtual address space for the loaded executable image. It should be a multiple of SectionAlignment.
; +     SizeOfHeaders.l ; How much space in the file is used for all the file headers, including the MS-DOS header, PE file header, PE optional header, and the sections headers.
; +     CheckSum.l
; +     Subsystem.w
; +     DllCharacteristics.w
; +     SizeOfStackReserve.i
; +     SizeOfStackCommit.i
; +     SizeOfHeapReserve.i
; +     SizeOfHeapCommit.i
; +     LoaderFlags.l
; +     NumberOfRvaAndSizes.l ; The number of the following data directory entries (should always be 16, but could change in the future).
; +- [IMAGE_DATA_DIRECTORY] ----------------------------------------------------+ repeated x [IMAGE_OPTIONAL_HEADER].NumberOfRvaAndSizes
; +     VirtualAddress.l -> RVA of the first structure associated with this specific directory entry (IMAGE_EXPORT_DIRECTORY, IMAGE_IMPORT_DESCRIPTOR, etc.)
; +     Size.l
; +-----------------------------------------------------------------------------+

;                              SECTION HEADERS 
; +- [IMAGE_SECTION_HEADER] ----------------------------------------------------+ repeated x [IMAGE_FILE_HEADER].NumberOfSections 
; +     SecName.b[8]
; +     StructureUnion
; +      PhysicalAddr.l
; +      VirtualSize.l ; The total size of the section when loaded into memory. If this value is greater than SizeOfRawData, the section is zero-padded. 
; +     EndStructureUnion
; +     VirtualAddress.l ; For executable images, the RVA of the first byte of the section.
; +     SizeOfRawData.l ; The size of the initialized data on disk. When a section contains only uninitialized data, this field should be zero.
; +     PointerToRawData.l ; The file pointer to the first page of the section. For executable images, this must be a multiple of FileAlignment from the optional header.
; +     PointerToRelocations.l
; +     PointerToLinenumbers.l
; +     NumberOfRelocations.w
; +     NumberOfLinenumbers.w
; +     Characteristics.l
; +-----------------------------------------------------------------------------+

;                                  SECTIONS 
; +-----------------------------------------------------------------------------+ repeated x [IMAGE_FILE_HEADER].NumberOfSections 
; + Various sections follow here, the contents of the raw data is depending on 
; + the section's type.
; + Some of the most notable are listed below ...
; +-----------------------------------------------------------------------------+

;                                IMPORT SECTION 
;                         (MS usually call it .idata)
; +- [IMAGE_IMPORT_DESCRIPTOR] -------------------------------------------------+ repeated x n times (zero terminated)
; +  OriginalFirstThunk.l -> points to an array of IMAGE_THUNK_DATA
; +  EndStructureUnion
; +  TimeDateStamp.l
; +  ForwarderChain.l
; +  Name.l
; +  FirstThunk.l
; +
; + The array is terminated by a IMAGE_IMPORT_DESCRIPTOR containing all zeroes.
; +-----------------------------------------------------------------------------+

;                                EXPORT SECTION 
;                          (MS usually call it .edata)
; +-----------------------------------------------------------------------------+
;                                     ...
; +-----------------------------------------------------------------------------+

;                              RELOCATION SECTION 
;                          (MS usually call it .reloc)
; +-----------------------------------------------------------------------------+
;                                     ...
; +-----------------------------------------------------------------------------+

;}

;EnableExplicit

Declare.i    MemDLL_GetProcAddress (hdll, func$)
Declare      MemDLL_FreeLibrary (hdll)
Declare.i    MemDLL_LoadLibrary (*image)

XIncludeFile "MemDll.pb"
; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 200
; FirstLine = 168
; Folding = -
; EnableUnicode
; EnableXP